// Comm.h : header file
//

/////////////////////////////////////////////////////////////////////////////
// CComm
/*

  genric serial port class, no HW handshaking or xon/xoff, 8-bit no parity

  use with Visual C++ 4.x or 5.x

  For 4.x, bool automatically is redefined as BOOL

  Interface functions:

  bool OpenComm(int Port,int baud=9600)
	- open comm port #Port (1=COM1:) if baud is not specified, set to 9600
	returns true if ok

  GetHandle() - returns file handle to device, can be used to use WriteFile(),
	ReadFile() and other comm functions (i.e. SetCommState())

  int SendBlock(char * buff, int len))
  SendChar(int ch)
  int SendString(CString str)
	- Write data to serial port, returns # bytes written

  int GetLine(CString &s)
	- Read a string terminated by cr/lf (\r\n) CR is not needed
		both cr and lf not returned.
	  Returnd number ob bytes read (may be 0 if timeout)

  int GetBlock(void * buff,int len)
	- get data from serial port into buffer, returns # bytes read

  SetDTR(bool flag)
  SetRTS(bool flag)
	- changes state of DTR/RTS, initially on after open, flag=true
		turns on, false=off

  bool CheckCTS()
  bool CheckDCS()
  bool CheckDSR()
  bool CheckRing()
	- returns status of handshaking lines

  ReInitPort()
	- reinitializes serial port;

  CloseComm() - close serial port

  SetBreak()
	- controls transmit of break state (all 0's)

  To use:

	1.	Create CComm Class
	2.	Call OpenComm
	3.	use Get/Send calls to rx/tx data
	4.	Call CloseComm when done

*/


#ifndef __COMM_H__
#define __COMM_H__
//UINT RxThread(LPVOID pParam);	// define thread

#if _MSC_VER < 1100
#define bool BOOL
#define false FALSE
#define true TRUE
#endif

class CComm 
{
// Construction
public:
	CComm();
	virtual ~CComm();

// Attributes
public:

// Operations
public:
bool OpenComm(int Port,int baud=9600,bool flow=false);

// Overrides

// Implementation
public:
	void PurgeRX();
	void WaitTX();
	bool m_FlowOn;
	void SetBreak(bool flag);
	void ReInitPort();
	int SendBlock(void * buff,int len);
	void SetBaud(int baud);
	void SetDTR(bool flag);
	void CloseComm();
	int SendString(CString str);
	int SendChar(int ch);
	void SetRTS(bool flag);
	bool CheckRing();
	bool CheckDSR();
	bool CheckCTS();
	int GetBlock(void * buff,int len);
	bool CheckDCD();
	bool GetLine(CString& s);
	int GetBaud();
	HANDLE GetHandle();
	bool IsActive();

protected:
	HANDLE hComm;
	DCB DCB;
	int Baud;
private:
};

/////////////////////////////////////////////////////////////////////////////
#endif